#include "CAN_S88_Detector.h"	// Detector program

#include <Arduino.h>
#include <SPI.h>      //  SPI library
#include <mcp_can.h>  // MCP2515 library

#define RETRY_CONNECTION 10    // retry connection

unsigned char dataHigh = 0;            // data from sensors 15 to 8
unsigned char dataLow = 0;             // data from sensors 7 to 1
unsigned char data[8];                 // gateway number,detector number,dataHigh,dataLow

// CAN parameters

const byte CS = 10;                     // NANO chip select
const int baudrate = CAN_500KBPS;       // can throughput 500 Kb/s  
MCP_CAN can(CS);                        // CAN instance
unsigned char gatewayNumber;            // number of associated gateway (1 to 8)
unsigned char detectorNumber;           // number of the detector (1 to 4)

// seed for random

byte getSeed() {
  byte seed = 0;
  byte i = 8;
  long timeLimit = millis();
  while (i>0) {
    byte firstBit = analogRead(A6)& 0x01;
    byte secondBit = analogRead(A7) & 0x01;
    if (firstBit != secondBit) {seed = (seed <<1) | firstBit; i--;}
    if (millis() - timeLimit > 10 ) {return seed;}
  }
 return seed;
}

// setup

void Setup(unsigned char IdentGateway,unsigned char IdentDetector) {
    detectorNumber = IdentDetector;                          // detector number
    gatewayNumber = IdentGateway;                            // gateway number
    int repeat = RETRY_CONNECTION;                           // try to open the CAN
    while (repeat > 0) {
        if (CAN_OK == can.begin(baudrate)){break;}
        else {repeat--;}
        delay(200);
    }

    // set of pins for sensors
    pinMode(0,INPUT_PULLUP);                            // pin RX
    for (int i =2;i<10;i++) {pinMode(i,INPUT_PULLUP);}  // pins 2 to 9
    for (int i =14;i<20;i++) {pinMode(i,INPUT_PULLUP);} // pins A0 to A5
    randomSeed(getSeed());
}

// Loop

void Loop() {

    delay(random(1,5));
  
    // sensors reset
    dataHigh = 0;
    dataLow = 0;
    
    // for sensors from 1 to 7
    if (!digitalRead(0)) {bitSet(dataLow,0);}
    for (int i=2;i<9;i++) {if (!digitalRead(i)){bitSet(dataLow,i-1);}}
    // for sensors from 8 to 15
    if (!digitalRead(9)){bitSet(dataHigh,0);}
    for (int i=14;i<20;i++) {if (!digitalRead(i)){bitSet(dataHigh,i-13);}}
    
    // send message
    data[0] = gatewayNumber;
    data[1] = detectorNumber;
    data[2] = dataHigh;
    data[3] = dataLow;
    can.sendMsgBuf(0x01,0,4,data);
    
}
